@@ -24,7 +24,7 @@
 # Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ##################################################################################
 
-import logging, os, os.path, signal, sys, re
+import logging, os, os.path, signal, re
 
 from gi.repository import GLib
 import dbus
@@ -39,15 +39,12 @@ import datetime
 import distutils.dir_util
 import lsb_release
 from hashlib import md5
-from aptdaemon import client
-from defer import inline_callbacks
 
 from Dell.recovery_common import (DOMAIN, LOCALEDIR,
                                   walk_cleanup, create_new_uuid, white_tree,
                                   black_tree, fetch_output, check_version,
                                   DBUS_BUS_NAME, DBUS_INTERFACE_NAME,
                                   RestoreFailed, CreateFailed,
-                                  mark_upgrades, mark_unconditional_debs,
                                   regenerate_md5sum, PermissionDeniedByPolicy)
 from Dell.recovery_threading import ProgressByPulse, ProgressBySize
 from Dell.recovery_xml import BTOxml
@@ -59,20 +56,7 @@ import select
 from gettext import gettext as _
 from gettext import bindtextdomain, textdomain
 
-#TODO, when python3 version of debian-support is available,
-# drop this ugly test
-if sys.version >= "3":
-    class emulator:
-        def version_compare(self, this, that):
-            if this > that:
-                return 1
-            if this == that:
-                return 0
-            else:
-                return -1
-    debian_support = emulator()
-else:
-    from debian_bundle import debian_support
+from debian_bundle import debian_support
 
 def safe_tar_extract(filename, destination):
     """Safely extracts a tarball into destination"""
@@ -363,7 +347,7 @@ class Backend(dbus.service.Object):
                 nested = False
                 rfd = tarfile.open(fishie)
                 for member in rfd.getmembers():
-                    name = member.get_info(encoding='UTF-8', errors='strict')['name']
+                    name = member.get_info()['name']
                     if name.endswith('.html'):
                         nested = name
                         break
@@ -435,7 +419,7 @@ class Backend(dbus.service.Object):
         self.main_loop.quit()
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = 'sasa{ss}ssss', out_signature = '', sender_keyword = 'sender',
+        in_signature = 'sasa{ss}sssss', out_signature = '', sender_keyword = 'sender',
         connection_keyword = 'conn')
     def assemble_image(self,
                        base,
@@ -443,18 +427,19 @@ class Backend(dbus.service.Object):
                        application_fish,
                        dell_recovery_package,
                        create_fn,
-                       version, iso, sender=None, conn=None):
+                       version, iso, platform, sender=None, conn=None):
         """Assemble pieces that would be used for building a BTO image.
            base: mount point of base image (or directory)
            fish: list of packages to fish
            dell_recovery_package: a dell-recovery package to inject
            create_fn: function to call for creation of ISO
            version: version for ISO creation purposes
-           iso: iso file name to create"""
+           iso: iso file name to create
+           platform: platform name to identify"""
         logging.debug("assemble_image: base %s, driver_fish %s, application_fish\
-%s, recovery %s, create_fn %s, version %s, iso %s" %
+%s, recovery %s, create_fn %s, version %s, iso %s, platform %s" %
                     (base, driver_fish, application_fish, dell_recovery_package,
-                    create_fn, version, iso))
+                    create_fn, version, iso, platform))
 
         self._reset_timeout()
 
@@ -514,10 +499,10 @@ class Backend(dbus.service.Object):
                 distutils.file_util.copy_file(dell_recovery_package, dest)
 
         function = getattr(Backend, create_fn)
-        function(self, assembly_tmp, version, iso)
+        function(self, assembly_tmp, version, iso, platform)
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = 's', out_signature = 'sssss', sender_keyword = 'sender',
+        in_signature = 's', out_signature = 'ssssss', sender_keyword = 'sender',
         connection_keyword = 'conn')
     def query_iso_information(self, iso, sender=None, conn=None):
         """Queries what type of ISO this is.  This same method will be used regardless
@@ -546,7 +531,7 @@ class Backend(dbus.service.Object):
         self._check_polkit_privilege(sender, conn,
                                 'com.dell.recoverymedia.query_iso_information')
 
-        (bto_version, bto_date) = self.query_bto_version(iso, sender, conn)
+        (bto_version, bto_date, bto_platform) = self.query_bto_version(iso, sender, conn)
 
         distributor_str = 'Unknown Base Image'
         distributor = ''
@@ -591,13 +576,13 @@ class Backend(dbus.service.Object):
         else:
             bto_version = ''
 
-        self.report_iso_info(bto_version, distributor, release, arch, distributor_str)
+        self.report_iso_info(bto_version, distributor, release, arch, distributor_str, bto_platform)
         logging.debug(" returning bto_version %s, distributor %s, release %s, \
-arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distributor_str))
-        return (bto_version, distributor, release, arch, distributor_str)
+arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, release, arch, distributor_str, bto_platform))
+        return (bto_version, distributor, release, arch, distributor_str, bto_platform)
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = 's', out_signature = 'ss', sender_keyword = 'sender',
+        in_signature = 's', out_signature = 'sss', sender_keyword = 'sender',
         connection_keyword = 'conn')
     def query_bto_version(self, recovery, sender=None, conn=None):
         """Queries the BTO version number internally stored in an ISO or RP"""
@@ -627,13 +612,16 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         #mount the recovery partition
         version = ''
         date = ''
+        platform = ''
 
         if os.path.isfile(recovery) and recovery.endswith('.iso'):
             cmd = ['isoinfo', '-J', '-i', recovery, '-x', '/bto.xml']
             out = fetch_output(cmd)
             if out:
                 self.xml_obj.load_bto_xml(out)
-                version = self.xml_obj.fetch_node_contents('iso')
+                version = self.xml_obj.fetch_node_contents('revision') or \
+                          self.xml_obj.fetch_node_contents('iso')
+                platform = self.xml_obj.fetch_node_contents('platform')
                 date = self.xml_obj.fetch_node_contents('date')
             else:
                 cmd = ['isoinfo', '-J', '-i', recovery, '-x', '/bto_version']
@@ -650,7 +638,9 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
             mntdir = self.request_mount(recovery, "r", sender, conn)
             if os.path.exists(os.path.join(mntdir, 'bto.xml')):
                 self.xml_obj.load_bto_xml(os.path.join(mntdir, 'bto.xml'))
-                version = self.xml_obj.fetch_node_contents('iso')
+                version = self.xml_obj.fetch_node_contents('revision') or \
+                          self.xml_obj.fetch_node_contents('iso')
+                platform = self.xml_obj.fetch_node_contents('platform')
                 date = self.xml_obj.fetch_node_contents('date')
             elif os.path.exists(os.path.join(mntdir, 'bto_version')):
                 with open(os.path.join(mntdir, 'bto_version'), 'r') as rfd:
@@ -660,7 +650,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
             elif os.path.exists(os.path.join(mntdir, 'casper', 'initrd.lz')):
                 version = test_initrd(['cat', os.path.join(mntdir, 'casper', 'initrd.lz')])
 
-        return (version, date)
+        return (version, date, platform)
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
         in_signature = 's', out_signature = 's', sender_keyword = 'sender',
@@ -787,168 +777,16 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         self.report_package_info(valid, description, error_warning)
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = 'ss', out_signature = '', sender_keyword = 'sender',
+        in_signature = 'b', out_signature = '', sender_keyword = 'sender',
         connection_keyword = 'conn')
-    def install_driver_package(self, package, recovery, sender=None, conn=None):
-        """Prepares to tnstall a Dell driver package"""
-        self._reset_timeout()
-        self._check_polkit_privilege(sender, conn, 'com.dell.recoverymedia.driverinstall')
-
-        #extract package
-        self.report_progress("Creating temporary directory.")
-        self.package_dir = tempfile.mkdtemp()
-        os.chmod(self.package_dir, 0o755)
-        atexit.register(walk_cleanup, self.package_dir)
-        self.report_progress("Extracting package.")
-        safe_tar_extract(package, self.package_dir)
-
-        #emulate fat32 partition automatically setting sh and py scripts executable
-        for root, dirs, files in os.walk(self.package_dir, topdown=False):
-            for name in files:
-                if name.endswith('.sh'):
-                    with open(os.path.join(root, name), 'r') as rfd:
-                        script = rfd.readlines()
-                    with open(os.path.join(root, name), 'w') as wfd:
-                        for line in script:
-                            if '/cdrom' in line:
-                                line = line.replace('/cdrom', self.package_dir)
-                            elif 'apt-get purge' in line:
-                                line = line.replace('\n', ' || true\n')
-                            wfd.write(line)
-                if name.endswith('.sh') or name.endswith('.py'):
-                    os.chmod(os.path.join(root, name), 0o755)
-
-        #mount RP (if possible)
-        if recovery:
-            self.report_progress("Mounting recovery partition.")
-            mntdir = self.request_mount(recovery, "r")
-            #build the pool with stuff from RP
-            for directory in ['debs', 'pool']:
-                if os.path.exists(os.path.join(mntdir, directory)):
-                    if not os.path.isdir(os.path.join(self.package_dir, 'debs')):
-                        os.mkdir(os.path.join(self.package_dir, 'debs'))
-                    os.symlink(os.path.join(mntdir, directory), \
-                        os.path.join(self.package_dir, 'debs', 'rp_' + directory))
-
-        #produce apt source
-        #if we have an apt source, this is taken over by signals now
-        #if not, then we jump directly to the next step
-        debs_dir = os.path.join(self.package_dir, 'debs')
-        if os.path.exists(debs_dir):
-            self.report_progress("Building APT Package listing.")
-            build = subprocess.Popen(['apt-ftparchive', 'packages', '.'],
-                             stdout=subprocess.PIPE,
-                             cwd=debs_dir)
-            output = build.communicate()[0]
-            with open(os.path.join(debs_dir, 'Packages'), 'wb') as wfd:
-                wfd.write(output)
-
-            source_list = os.path.join('/etc', 'apt', 'sources.list.d', os.path.basename(self.package_dir) + '.list')
-            if os.path.exists(source_list):
-                raise CreateFailed("File already exists")
-            with open(source_list, 'w') as wfd:
-                wfd.write("deb file:%s /\n" % debs_dir)
-            atexit.register(walk_cleanup, source_list)
-
-            self._update_cache(source_list)
-        else:
-            self._process_scripts('exit-success')
-
-    @inline_callbacks
-    def _install_debs(self):
-        """Use AptDaemon to install all debs that would normally be installed"""
-        logging.debug('enter _install_debs')
-        apt_client = client.AptClient()
-        to_install = mark_upgrades() + mark_unconditional_debs(self.package_dir)
-        logging.debug("debs to install: %s" % to_install)
-        tid = yield apt_client.install_packages(to_install)
-        yield tid.set_allow_unauthenticated(True)
-        yield tid.connect("finished", self._install_debs_finished)
-        self.report_progress("Installing debian packages.", tid.tid)
-        yield tid.run()
-        logging.debug("leave _install_debs")
-
-    @inline_callbacks
-    def _update_cache(self, source_list):
-        """Use AptDaemon to update the partial cache"""
-        logging.debug("enter _update_cache")
-        apt_client = client.AptClient()
-        tid = yield apt_client.update_cache(source_list)
-        yield tid.connect("finished", self._update_cache_finished)
-        self.report_progress("Building APT cache.", tid.tid)
-        yield tid.run()
-        logging.debug("leave _update_cache")
-
-    def _update_cache_finished(self, trans, exit_status):
-        """Signal relay to notify frontend that the cache is ready to go"""
-        logging.debug("Cache update completed. exit status: %s" % exit_status)
-        #if not success, return now
-        if exit_status != 'exit-success':
-            self.report_package_installed(exit_status, '')
-        #otherwise continue and try to install debs
-        self._install_debs()
-
-    def _install_debs_finished(self, trans, exit_status):
-        """Signal relay for debs being finished installed (or none present)"""
-        logging.debug("Deb install completed. exit status %s" % exit_status)
-        #if not success, return now
-        if exit_status != 'exit-success':
-            self.report_package_installed(exit_status, '')
-        #otherwise continue and try to process scripts
-        try:
-            self._process_scripts(exit_status)
-        except RuntimeError as msg:
-            self.report_package_installed('exit-failed', str(msg))
-
-    def _process_scripts(self, exit_status):
-        """Process the scripts portion of a FISH package"""
-        logging.debug("enter _process_scripts")
-        scripts = []
-        self.report_progress("Processing post scripts.", '')
-        for directory in (os.path.join(self.package_dir, 'scripts',
-                                       'chroot-scripts', 'fish'),
-                          '/usr/share/dell/scripts/non-negotiable'):
-            if os.path.isdir(directory):
-                for item in os.listdir(directory):
-                    scripts.append(os.path.join(directory, item))
-        environment = os.environ
-        environment['DEBIAN_FRONTEND'] = 'noninteractive'
-        environment['PATH'] = '/sbin:/usr/sbin:/bin:/usr/bin'
-        scripts.append('/usr/bin/apt-get -f install --yes')
-        scripts.append('/usr/bin/ubuntu-drivers autoinstall')
-        num = float(1)
-        total = len(scripts)
-        for script in scripts:
-            if 'installer' in script:
-                logging.debug('skipping script: %s' % script)
-                continue
-            logging.debug('executing script: %s' % script)
-            self.report_progress("executing %s" % script, num/total)
-            #in case shell script is missing starting shebang (a few are)
-            if script.endswith('.sh'):
-                cmd = ['/bin/sh', script]
-            else:
-                #for apt-get and ubuntu-drivers
-                if ' ' in script:
-                    cmd = script.split()
-                else:
-                    cmd = [script]
-            logging.debug(fetch_output(cmd, None, environment))
-            num+=1
-        logging.debug("leave _process_scripts")
-        self.report_package_installed(exit_status, '')
-
-    @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = '', out_signature = '', sender_keyword = 'sender',
-        connection_keyword = 'conn')
-    def enable_boot_to_restore(self, sender=None, conn=None):
+    def enable_boot_to_restore(self, reboot, sender=None, conn=None):
         """Enables the default one-time boot option to be recovery"""
         self._reset_timeout()
         self._check_polkit_privilege(sender, conn, 'com.dell.recoverymedia.restore')
-        logging.debug("enable_boot_to_restore")
-        self._prepare_reboot("99_dell_recovery")
+        logging.debug("enable_boot_to_restore:reboot=%s" % reboot)
+        self._prepare_reboot("99_dell_recovery", reboot)
 
-    def _prepare_reboot(self, dest):
+    def _prepare_reboot(self, dest, reboot):
         """Helper function to reboot into an entry"""
         #find our one time boot entry
         if not os.path.exists("/etc/grub.d/%s" % dest):
@@ -993,18 +831,24 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         if ret is not 0:
             raise RestoreFailed("error setting one time grub entry")
 
+        if reboot:
+            logging.debug("Prepare to reboot")
+            ret = subprocess.call(["/sbin/reboot", "--force"])
+            if ret is not 0:
+                raise RestoreFailed("error invoking reboot")
+
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
-        in_signature = 'sss', out_signature = '', sender_keyword = 'sender',
+        in_signature = 'ssss', out_signature = '', sender_keyword = 'sender',
         connection_keyword = 'conn')
-    def create_ubuntu(self, recovery, version, iso, sender=None, conn=None):
+    def create_ubuntu(self, recovery, revision, iso, platform, sender=None, conn=None):
         """Creates Ubuntu compatible recovery media"""
 
         self._reset_timeout()
         self._check_polkit_privilege(sender, conn,
                                                 'com.dell.recoverymedia.create')
-        logging.debug("create_ubuntu: recovery %s, version %s, iso %s" %
-            (recovery, version, iso))
+        logging.debug("create_ubuntu: recovery %s, revision %s, iso %s, platform %s" %
+            (recovery, revision, iso, platform))
 
         #create temporary workspace
         tmpdir = tempfile.mkdtemp()
@@ -1040,7 +884,9 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
 
         #Generate BTO XML File
         self.xml_obj.replace_node_contents('date', str(datetime.date.today()))
-        self.xml_obj.replace_node_contents('iso', version)
+        self.xml_obj.replace_node_contents('iso', os.path.basename(iso))
+        self.xml_obj.replace_node_contents('revision', revision)
+        self.xml_obj.replace_node_contents('platform', platform)
         self.xml_obj.replace_node_contents('generator', check_version())
         self.xml_obj.write_xml(os.path.join(tmpdir, 'bto.xml'))
 
@@ -1052,8 +898,9 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                        '-J',
                        '-joliet-long',
                        '-l',
-                       '-cache-inodes',
                        '-iso-level', '3',
+                       '-sysid', 'Linux',
+                       '-V', 'Dell Recovery Media',
                        '-A', 'Dell Recovery',
                        '-p', 'Dell',
                        '-publisher', 'Dell',
@@ -1063,20 +910,33 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                        '-m', 'syslinux',
                        '-m', 'syslinux.cfg',
                        '-m', os.path.join(mntdir, 'bto.xml'),
-                       '-m', os.path.join(mntdir, 'isolinux'),
                        '-m', os.path.join(mntdir, 'bto_version')]
 
+        if platform and revision:
+            xorrisoargs.append('-volset')
+            xorrisoargs.append(platform + ' ' + revision)
+
+        if os.path.exists(os.path.join('/', 'usr', 'lib', 'ISOLINUX', 'isohdpfx.bin')) and \
+            os.path.exists(os.path.join(mntdir, 'isolinux', 'boot.cat')) and \
+            os.path.exists(os.path.join(mntdir, 'isolinux', 'isolinux.bin')):
+            xorrisoargs.append('-c')
+            xorrisoargs.append('isolinux/boot.cat')
+            xorrisoargs.append('-b')
+            xorrisoargs.append('isolinux/isolinux.bin')
+            xorrisoargs.append('-no-emul-boot')
+            xorrisoargs.append('-boot-load-size')
+            xorrisoargs.append('4')
+            xorrisoargs.append('-boot-info-table')
+            xorrisoargs.append('-isohybrid-mbr')
+            xorrisoargs.append(os.path.join('/', 'usr', 'lib', 'ISOLINUX', 'isohdpfx.bin'))
+
         #include bootloader as eltorito if we have it
         if os.path.exists(os.path.join(mntdir, 'boot', 'efi.img')):
+            xorrisoargs.append('-eltorito-alt-boot')
             xorrisoargs.append('-e')
             xorrisoargs.append('boot/efi.img')
             xorrisoargs.append('-no-emul-boot')
-            xorrisoargs.append('-append_partition')
-            xorrisoargs.append('2')
-            xorrisoargs.append('0xef')
-            xorrisoargs.append(os.path.join(mntdir, 'boot', 'efi.img'))
-            xorrisoargs.append('-partition_cyl_align')
-            xorrisoargs.append('all')
+            xorrisoargs.append('-isohybrid-gpt-basdat')
 
         #disable 32 bit bootloader if it was there.
         grub_path = os.path.join(mntdir, 'boot', 'grub', 'i386-pc')
@@ -1214,7 +1074,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                                output.strip())
 
     @dbus.service.signal(DBUS_INTERFACE_NAME)
-    def report_iso_info(self, version, distributor, release, arch, output_text):
+    def report_iso_info(self, version, distributor, release, arch, output_text, platform):
         '''Report ISO information to UI.
         '''
         return True
