@@ -31,7 +31,6 @@ from Dell.recovery_threading import ProgressBySize
 import debconf
 import Dell.recovery_common as magic
 from Dell.recovery_xml import BTOxml
-import subprocess
 import os
 import re
 import shutil
@@ -142,7 +141,6 @@ class PageGtk(PluginUI):
             icon = builder.get_object('dell_image')
             icon.set_tooltip_markup("Dell Recovery Advanced Options")
             self.advanced_page = builder.get_object('advanced_window')
-            self.advanced_table = builder.get_object('advanced_table')
             self.version_detail = builder.get_object('version_detail')
             self.mount_detail = builder.get_object('mountpoint_detail')
             self.memory_detail = builder.get_object('memory_detail')
@@ -158,7 +156,6 @@ class PageGtk(PluginUI):
         """
         #are we real?
         if not (self.genuine and 'UBIQUITY_AUTOMATIC' in os.environ):
-            self.advanced_table.set_sensitive(False)
             self.interactive_recovery_box.hide()
             self.automated_recovery_box.hide()
             self.automated_recovery.set_sensitive(False)
@@ -209,7 +206,6 @@ class PageGtk(PluginUI):
         else:
             self.controller.allow_go_forward(False)
             if value == "hdd":
-                self.advanced_table.set_sensitive(False)
                 self.hdd_recovery_box.show()
                 self.interactive_recovery_box.hide()
                 self.automated_recovery_box.hide()
@@ -363,6 +359,13 @@ class Page(Plugin):
 
     def test_swap(self):
         """Tests whether to do a swap fixup"""
+        import lsb_release
+        release = lsb_release.get_distro_information()
+
+        #starting with 17.04, we replace the whole swap partition to swap file
+        if float(release["RELEASE"]) >= 17.04:
+            return True
+
         if (self.mem >= 32 or self.disk_size <= 64):
             return True
         else:
@@ -390,7 +393,7 @@ class Page(Plugin):
 
        # check dual boot or not
         try:
-            if self.db.get('dell-recovery/dual_boot')=='true':
+            if self.db.get('dell-recovery/dual_boot') == 'true':
            ##dual boot get the partition number of OS and swap
                 os_label = self.db.get('dell-recovery/os_partition')
                 os_part,swap_part = self.dual_partition_num(os_label)
@@ -535,22 +538,10 @@ class Page(Plugin):
             #Check the disk is type of NVME SSD
             device_path = block.get_cached_property("Device").get_bytestring().decode('utf-8')
             if device_path.startswith('/dev/nvme'):
-                with misc.raised_privileges():
-                    getdisk_subp = subprocess.Popen(['smartctl', '-d', 'scsi', '--all', device_path], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-                    output = getdisk_subp.communicate()[0]
-                    vendor = ""
-                    model = ""
-                    for prop in output.decode('utf-8').split('\n'):
-                        if prop.startswith('Vendor:'):
-                            vendor = prop.split(':')[1].strip()
-                        elif prop.startswith('Product:'):
-                            model = prop.split(':')[1].strip()
-                        else:
-                            continue
-
+                output = block.get_cached_property("Id").get_string()
+                model = output.split("-")[-1].replace("_", " ")
                 nvme_dev_size = block.get_cached_property("Size").unpack()
-                nvme_size_gb = "%i" % (nvme_dev_size / 1000000000)
-                disks.append([device_path, nvme_dev_size, "%s GB %s %s (%s)" % (nvme_size_gb, vendor, model, device_path)])
+                disks.append([device_path, nvme_dev_size, "%s (%s)" % (model, device_path)])
                 continue
 
             drive_obj = block.get_cached_property("Drive").get_string()
@@ -1235,6 +1226,15 @@ class Install(InstallPlugin):
                 if not found:
                     wfd.write("GRUB_DISABLE_OS_PROBER=true\n")
 
+            #set default recovery_type of 99_dell_recovery grub as 'hdd' for non-Wyse platforms
+            recovery_type = 'hdd'
+            #if wyse mode is on (dell-recovery/mode == 'wyse'), set the recovery_type to be 'factory'
+            #as Wyse platforms will always skip the "Restore OS Linux partition" dialog
+            if self.db.get('dell-recovery/wyse_mode') == 'true':
+                recovery_type = 'factory'
+            #create 99_dell_recovery grub
+            magic.create_grub_entries(self.target, recovery_type)
+
         #for tos
         try:
             destination = progress.get('dell-recovery/destination')
@@ -1254,4 +1254,8 @@ class Install(InstallPlugin):
         install_misc.record_installed(to_install)
         install_misc.record_removed(to_remove)
 
+        #copy the to_install package list into /tmp as backup to check
+        apt_installed = "/var/lib/ubiquity/apt-installed"
+        shutil.copy(apt_installed, os.path.join(self.target, 'tmp', os.path.basename(apt_installed)))
+
         return InstallPlugin.install(self, target, progress, *args, **kwargs)
